/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __AUDIODIV_H
#define __AUDIODIV_H


#include <math.h>
#include "global.h"
#include "reverb.h"


extern float exp2ap (float);


class Tremul
{
public:

    Tremul (float conv) : _f (4.0), _m (0.4), _stat (0), _conv (conv), _c (1), _s (0) {}
    void set_on (void)  { _stat = 1; }
    void set_off (void) { _stat = 2; }
    float step (void)
    {
        float w, t;

        w  = _f * _conv;
        if (_stat)
	{
            _s += w * _c;
	    _c -= w * _s;
            t = sqrt (_c * _c + _s * _s);
            _c /= t;
            _s /= t;
            if ((_stat == 2) && (fabs (_s) < 0.01))
	    {
		_stat = 0;
                _c = 1;
                _s = 0;
	    }
	}
        return 1.0 + _m * _s;
    }        

    float  _f;
    float  _m;

private:
    int    _stat;
    float  _conv;
    float  _c;
    float  _s;

};



class Audiodiv
{
public:

    Audiodiv (float fsamp);
    ~Audiodiv (void); 

    float *base (void) { return _base; }
    int    offs (void) { return _offs [0]; } 

    float   _dir_g;
    float   _dir_w;
    float   _rfl_g;
    float   _rev_g;
    Tremul  _tremul;

    void set_swell (float v) { _targ = exp2ap (3.0 * (v - 1.0)); }
    void set_rsize (float size, float fsam, float *list);
    void process (float vol, float *W, float *X, float *Y, float *R);

    static float _refl0 [16];
    static float _refl1 [16];
    static float _refl2 [16];

private:

    int      _offs [20];
    float   *_base;
    float    _targ;
    float    _swel;
    float    _gain;
    Diffuser _difw;
    Diffuser _difx;
    Diffuser _dify;
};


#endif

