/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __MAINWIN_H
#define __MAINWIN_H


#include <clxclient.h>
#include "def_addsynth1.h"
#include "editwin.h"
#include "audiowin.h"
#include "calcthr.h"
#include "audiodiv.h"
#include "midimatrix.h"


#define NDIVIS 4
#define NSTOPS 24


class Stopdata
{
public:

    Stopdata::Stopdata (void);
    void set_stopname (void);

    char             _text [16];
    int              _func;
    Def_addsynth1   *_stop;
    X_button        *_butt;
    int              _pan;
    int              _del;
};


class Division
{
public:

    Division::Division (void);

    char            _name [16];
    int             _ncnt;
    int             _ypos;
    int             _size;
    int             _ndef;
    Stopdata        _stop [NSTOPS];
};



class Mainwin : public X_window, public X_callback
{
public:

    Mainwin (X_window *parent, int xp, int yp, ITC_ctrl *athr, ITC_ctrl *xtrh, X_resman *xrm);
    ~Mainwin (void);
 
    virtual void handle_event (XEvent *);
    virtual void handle_callb (int, X_window*, _XEvent*);
    
    void handle_mesg (ITC_mesg *);
    void handle_time (void);

private:

    enum { TAB_SEL, TAB_MID, TAB_TUN, B_NEXT, B_PREV, B_AWIN, B_PRES, B_SAVE, B_MOFF,
           B_DECRB, B_INCRB, B_DECRM, B_INCRM, B_RECALL, B_MSTORE, B_INSERT, B_DELETE,
           B_TEMP_DEC, B_TEMP_INC, B_FREQ_DEC, B_FREQ_INC, B_TUNE_EXE, B_TUNE_CAN };
    enum { DIVBASE = 0x100, DIVSTEP = 0x100 };
    enum { VOLUM, RSIZE, RTIME };
    enum { DIFG, DRYS, REFL, REVB, TRFR, TRMD };

    void xcmesg (XClientMessageEvent *);
    void expose (XExposeEvent *);
    void initialise (const char *);
    void make_memory_tab (int, int);
    void make_mmatrx_tab (int, int);
    void make_tuning_tab (int, int);
    void make_audio_win ();
    void calculate (int, int);
    void calcdone (int, int);
    void set_tab (int);
    void set_mem (void);
    void set_temp (int);
    void set_freq (int);
    void add_text (X_window *win, int xp, int yp, int xs, int ys, const char *text, unsigned long fg);
    void retune (void);
    void save (void);


    ITC_ctrl      *_athr;
    ITC_ctrl      *_xthr;
    X_resman      *_xrm;
    Atom           _atom;
    char           _sdir [256];
    int            _xs;
    int            _ys;
    Division       _divisions [NDIVIS];
    unsigned int   _cmask [NDIVIS];
    unsigned int   _smask [NDIVIS];
    unsigned int  *_mchan;
    Editwin       *_editwin;
    Audiowin      *_audiowin;
    Calcthr       *_calcthr;
    X_button      *_flash;
    float          _fsamp;
    int            _nlock;
    int            _fstat;
    int            _ndiv;
    int            _ed_div;
    int            _ed_ind;
    X_button      *_tabb [3];
    X_window      *_tabw [3];
    int            _tabh [3];
    int            _ctab;
    X_button       *_awin;
    X_button       *_pres;
    X_button       *_save;
    X_button       *_moff;
    Midimatrix     *_mmat;  

    Audiodiv       *_adiv0;
    Audiodiv       *_adiv1;
    Audiodiv       *_adiv2;
    X_hslider      *_rsize;
    X_hslider      *_rtime;
    X_hslider      *_volum;

    X_hslider      *_difg0;
    X_hslider      *_drys0;
    X_hslider      *_refl0;
    X_hslider      *_revb0;
    X_hslider      *_difg1;
    X_hslider      *_drys1;
    X_hslider      *_refl1;
    X_hslider      *_revb1;
    X_hslider      *_difg2;
    X_hslider      *_drys2;
    X_hslider      *_refl2;
    X_hslider      *_revb2;
    float          *_apar;

    X_button       *_next;
    X_button       *_prev;
    X_button       *_decrb;
    X_button       *_incrb;
    X_textip       *_mdisp;
    X_button       *_decrm;
    X_button       *_incrm;
    X_button       *_recall;   
    X_button       *_mstore;   
    X_button       *_insert;   
    X_button       *_delete;   
    int            *_mpar;
    unsigned int   *_smem;
    X_button       *_dec_freq;
    X_button       *_inc_freq;
    X_button       *_dec_temp;
    X_button       *_inc_temp;
    X_button       *_tune_exe;
    X_button       *_tune_can;
    X_textip       *_freq_txt;
    X_textip       *_temp_txt;
    int             _cfreq;
    int             _nfreq;
    int             _ctemp;
    int             _ntemp;
    X_slider       *_trem0_freq;
    X_slider       *_trem0_ampl;
    X_slider       *_trem1_freq;
    X_slider       *_trem1_ampl;
};


#endif
