/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __STOPWAVE_H
#define __STOPWAVE_H


#include "def_addsynth1.h"


class Pipewave
{
public:
    enum { TERM = 256 };

    friend class Stopwave;   

private:


    Pipewave (void) : _link (0), _del (0), _p0 (0), _p2 (0), _l2 (0), _rl (1),
                      _rc (0), _rm (0), _rg (1), _pp (0), _rp (0) {}

    inline int play (float *out, int ind);

    void note_on (void)
    {
        _rc &= ~TERM;
        if (! _pp) _pp = _p0;
    }

    void note_off (void)
    {
        if (_pp) _rc |= TERM;
    }

    Pipewave  *_link;
    int     _del;
    float  *_p0;
    float  *_p2;
    int     _l2;
    short   _rl;
    short   _rc;
    float   _rm;
    float   _rg;    
    float  *_pp;    
    float  *_rp;
};


class Stopwave
{
public:

    Stopwave (int n);
    ~Stopwave (void);

    void note_on (int k)
    {
        Pipewave *P = _pipes + k;

        if (! (P->_pp || P->_rp)) { P->_link = _list; _list = P; }
        P->note_on ();   
    }

    void note_off (int k)
    {
        (_pipes + k)->note_off ();     
    }

    int  mask (void) { return _mask; }
    void stop_on (const unsigned char *notes, int mask);
    void stop_off (void);
    void play (float *out, int ind);
    void gen_waves (Def_addsynth1 *D, float fsamp, float fbase, float *scale, bool trace);
    void set_posit (int pan, int del);

private:

    Stopwave (const Stopwave&);
    Stopwave& operator=(const Stopwave&);

    void looplen (float f, float fsamp, int lmax, int *aa, int *bb);
    void attgain (int n, float *g, float p, bool trace);

    int         _npipe;
    float      *_waves;
    Pipewave   *_pipes;
    Pipewave   *_list;
    int         _mask;
};


#endif

